@@ -6,6 +6,7 @@ Mostly usefull during development and testing.
 __all__ = ['start_wsgi_server']
 
 from _wsgi import build_handler
+import _config as conf
 
 import logging
 logger = logging.getLogger('fanery.server')
@@ -28,6 +29,10 @@ def start_wsgi_server(wsgi_app=None, host='127.0.0.1', port=9000,
     if wsgi_app is None:
         wsgi_app = build_handler(**argd)
 
+    if argd.pop('gzip', conf.IS_DEVELOPMENT) is True:
+        from paste.gzipper import make_gzip_middleware
+        wsgi_app = make_gzip_middleware(wsgi_app, None)
+
     logger.info("serving on %s://%s:%s" % ('https' if ssl else 'http', host, port))     # noqa
 
     if ssl is True:
