@@ -78,22 +78,22 @@ class DataStore():
 
         # register auth hooks
 
-        if permission:
+        if permission is not None:
             hooks.has_permission = permission.has_permission
 
-        if settings:
+        if settings is not None:
             hooks.update((name, getattr(settings, name))
                          for name in default_settings
                          if hasattr(settings, name))
 
-        if profile:
+        if profile is not None:
             hooks.fetch_profile = profile.fetch_profile
 
-        if abuse:
+        if abuse is not None:
             hooks.update(incr_abuse=abuse.incr_abuse,
                          abuse_level_by_origin=abuse.abuse_level_by_origin)
 
-        if state:
+        if state is not None:
             hooks.update(load_state=state.load_state,
                          store_state=state.store_state,
                          destroy_state=state.destroy_state,
