@@ -15,8 +15,8 @@ from _term import (
     Hict, normalize_filename, parse_json, to_simple, to_json,
     is_file_path, is_dir_path, is_sequence, is_generator
 )
+from _service import consume, lookup, traceback
 from _autoreload import trigger_autoreload
-from _service import consume, traceback
 from _state import State
 from _exc import NotFound
 from _version import __version__
@@ -32,9 +32,9 @@ STRICT_HTML_HEADERS = {
     'Content-Security-Policy': "default-src 'self' 'unsafe-eval'",
 }
 
+from os.path import normpath, basename, join
 from tempfile import NamedTemporaryFile
 from os import link, unlink, chmod
-from os.path import basename, join
 from stat import S_IRUSR
 from cgi import FieldStorage
 
@@ -86,13 +86,16 @@ def handler(req):
     except:
         return HTTPBadRequest(ENCODING_ERROR, server=SERVER)
 
-    # parse params and consume service
+    # parse request and consume service
 
+    headers = {'Content-Security-Policy': "default-src 'none'"}
+    is_development = conf.IS_DEVELOPMENT
     uploads = set()
 
     try:
         argd, params, path, = Hict(), req.params, req.path
 
+        # parse request parameters
         for key in set(params.keys()):
             if not key.startswith('_'):
                 if key.endswith('[]'):
@@ -108,16 +111,29 @@ def handler(req):
         if '_json_' in params:
             argd.update(parse_json(params['_json_']))
 
+        # build request state
         _state_ = State(domain=req.host.split(':', 1)[0],
                         origin=req.client_addr or '127.0.0.1',
                         sid=None, profile=None, role=None,
                         ssl=bool(req.headers.get('X-Forwarded-Proto', req.scheme) == 'https'))                  # noqa
 
-        if conf.IS_DEVELOPMENT:
+        # trigger service reload
+        if is_development:
             trigger_autoreload()
 
-        fun, ext, ret = consume(_state_, path, **argd)
-        uploads.clear()   # keep uploaded files
+        # service lookup
+        fun, argv, ext, out = lookup(normpath(path))
+
+        # RFD and XSS attack mitigation against JSON/JSONP Web APIs
+        if fun.safe and not is_development:
+            assert req.headers.get('X-Fanery-Call', None) == 'true', 'api-call'
+            headers['Content-Disposition'] = 'attachment; filename=f.txt'
+
+        # perform service call
+        _, _, ret = consume(_state_, (path, fun, argv, ext, out), **argd)
+
+        # keep uploaded files
+        uploads.clear()
     except NotFound:
         return HTTPNotFound(server=SERVER)
     except NotImplementedError:
@@ -125,7 +141,7 @@ def handler(req):
     except UnicodeDecodeError:
         return HTTPBadRequest(ENCODING_ERROR, server=SERVER)
     except Exception, e:
-        if conf.IS_DEVELOPMENT: traceback.print_exc()   # noqa
+        if is_development: traceback.print_exc()   # noqa
         res = HTTPBadRequest(content_type='text/plain', server=SERVER)
         error = dict(exc=e.__class__.__name__, err=e.args)
         res.body = to_json(to_simple(error))
@@ -137,8 +153,6 @@ def handler(req):
 
     # build answer
 
-    headers = {'Content-Security-Policy': "default-src 'none'"}
-
     if fun.ssl is True:
         headers['Strict-Transport-Security'] = "max-age=31536000; includeSubDomains"                        # noqa
 
@@ -185,7 +199,7 @@ def handler(req):
             return HTTPNotFound(server=SERVER)
 
     else:
-        content_type = guess_type(path)[0] or CONTENT_TYPE
+        headers['Content-Type'] = content_type = guess_type(path)[0] or CONTENT_TYPE                        # noqa
 
         if content_type == 'text/html':
             headers.update(STRICT_HTML_HEADERS)
